"use strict";

require("core-js/modules/es.promise");

require("core-js/modules/es.string.replace");

exports.__esModule = true;
exports.onCreatePage = exports.resolvableExtensions = exports.preprocessSource = exports.onCreateWebpackConfig = void 0;

var _bsbJs = require("bsb-js");

var _fs = _interopRequireDefault(require("fs"));

var _util = require("util");

var _utils = require("./utils");

var _path = _interopRequireDefault(require("path"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const readFile = (0, _util.promisify)(_fs.default.readFile);
const BS_TEST = /\.(bs.js)$/;
const REASON_TEST = /\.(ml|re)$/;

const isCompiledFile = fileName => BS_TEST.test(fileName);

const isReasonFile = fileName => REASON_TEST.test(fileName);

const onCreateWebpackConfig = ({
  actions
}) => {
  const {
    setWebpackConfig
  } = actions;
  setWebpackConfig({
    module: {
      rules: [{
        test: REASON_TEST,
        use: [`bs-loader`]
      }]
    }
  });
};

exports.onCreateWebpackConfig = onCreateWebpackConfig;

const jsFilePath = (buildDir, moduleDir, resourcePath, inSource, bsSuffix) => {
  const mlFileName = resourcePath.replace(buildDir, '');
  const jsFileName = mlFileName.replace(REASON_TEST, bsSuffix);

  if (inSource) {
    return _path.default.join(buildDir, jsFileName);
  }

  return _path.default.join(buildDir, 'lib', moduleDir, jsFileName);
};

const preprocessSource = async ({
  filename
}) => {
  if (!isReasonFile(filename)) {
    return null;
  }

  const moduleDir = 'js';
  const compiledFilePath = jsFilePath(process.cwd(), moduleDir, filename, false, '.bs.js');

  try {
    await (0, _bsbJs.compileFile)(moduleDir, compiledFilePath);
  } catch (e) {
    // Don't need to print error message since bsb will already do that
    process.exit(1);
  }
};

exports.preprocessSource = preprocessSource;

const resolvableExtensions = () => ['.ml', '.re'];

exports.resolvableExtensions = resolvableExtensions;

const onCreatePage = async ({
  page,
  actions: {
    createPage,
    deletePage
  }
}, {
  derivePathFromComponentName
}) => {
  const oldPage = Object.assign({}, page);
  const {
    component,
    path
  } = page;

  if (isCompiledFile(component)) {
    // Remove .bs components so we don't have duplicates
    deletePage(oldPage);
  } else if (derivePathFromComponentName && isReasonFile(component) && !path.endsWith('.html')) {
    // Try to grab the name of the component from the ReasonReact
    // component instead of using the file name
    const source = await readFile(component, 'utf-8');
    const newPath = (0, _utils.getPathForComponent)(source);

    if (newPath !== undefined) {
      const newPage = Object.assign({}, page, {
        path: newPath
      });
      deletePage(oldPage);
      createPage(newPage);
    }
  }
};

exports.onCreatePage = onCreatePage;